/*
 * contains special implementation for message dialog box that contains hiperlinks
 */

#define IDC_SYSLINK			15990

INT_PTR CALLBACK SysLinkDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
BOOL OnInitSysLinkDialog(HWND hWnd, HWND hWndFocus, LPARAM lParam);
LRESULT OnSysLinkNotify(HWND hWnd, int idCtrl, LPNMHDR pNMHdr);
LRESULT OnSysLinkCommand(HWND hWnd, WPARAM wParam, HWND lParam,int bls);
void OnClose(HWND hWnd);

typedef struct EndMsgParam{
	char* String;
	HINSTANCE hInst;
}END_MSG_PARAM;

void AdjustDialogSize(HDC hdc,HWND hWnd, HWND hLink,WCHAR* String);

#define MAX_WIDTH 100

//
// used when the message has a link in it
//

HFONT         hFont  = NULL;
DWORD Version_1;
int NewMessageDisplay(HWND hWnd, HINSTANCE hInst,char* FinalMessageString,DWORD Version)
{
   END_MSG_PARAM endMsg;
   INT_PTR hDlg = -1;


   endMsg.String = FinalMessageString;
   endMsg.hInst = hInst;
   Version_1 = Version;

   // create dialog
   if( NULL != hWnd )
     ShowWindow(hWnd,SW_HIDE);

   hDlg = DialogBoxParam(hInst, 
                MAKEINTRESOURCE(IDD_SYSLINKDIALOG), 
                hWnd, SysLinkDlgProc, (LPARAM)&endMsg);
   if( hDlg == -1 ){
      return 0;
   }

   return 1;

}

//  FUNCTION: SysLinkDlgProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages for the SysLink control dialog.
//
//
INT_PTR CALLBACK SysLinkDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        // Handle the WM_INITDIALOG message in OnInitSysLinkDialog
        HANDLE_MSG (hWnd, WM_INITDIALOG, OnInitSysLinkDialog);

        // Handle the WM_CLOSE message in OnClose
        HANDLE_MSG (hWnd, WM_CLOSE, OnClose);

        // Handle the WM_NOTIFY message in OnNotify
        HANDLE_MSG (hWnd, WM_NOTIFY, OnSysLinkNotify);
		HANDLE_MSG (hWnd, WM_COMMAND, OnSysLinkCommand);

    default:
        return FALSE;	// Let system deal with msg
    }
    return 0;
}

BOOL OnInitSysLinkDialog(HWND hWnd, HWND hWndFocus, LPARAM lParam)
{
    // Load and register SysLink control class
    INITCOMMONCONTROLSEX iccx;
    RECT rcParent;
    RECT rc = { 20, 20, 500, 100 };
    HWND hLink = NULL;
    char bluba[20]={{0}};
    END_MSG_PARAM* endMsg = (END_MSG_PARAM*)lParam;
    HMODULE hMod;
    WCHAR wCmdLine[1024];
    WCHAR fontName[1024];
    HDC hDC;
    TEXTMETRIC lptm; 

    iccx.dwSize = sizeof(INITCOMMONCONTROLSEX);
    iccx.dwICC = ICC_LINK_CLASS;



    InitCommonControlsEx(&iccx);


    // Create the SysLink control
    // The SysLink control supports the anchor tag(<a>) along with the 
    // attributes HREF and ID. 

    //
    // SYS link supports ONLY wide char strings -> we convert char to wchar
    MultiByteToWideChar( CP_ACP, 0, endMsg->String,strlen(endMsg->String)+1, wCmdLine, 1024 );

    hLink = CreateWindowExW(0, WC_LINK, wCmdLine,
        WS_VISIBLE | WS_CHILD | WS_TABSTOP, 
        rc.left, rc.top, rc.right, rc.bottom, 
        hWnd, (HMENU)IDC_SYSLINK, endMsg->hInst, NULL);
 
    if( hLink == NULL ){
       return FALSE;
    }

    /*
     * change font looks from default to non bold
     */
    {
		const INT     ITEM_POINT_SIZE = 16;
		LPTSTR        szFont = TEXT("MS Shell Dlg");
		OSVERSIONINFO osv    = { sizeof(OSVERSIONINFO) };
		HDC           hdc    = GetDC(hLink);
		INT           nFontHeight = MulDiv(ITEM_POINT_SIZE, GetDeviceCaps(hdc, LOGPIXELSY), 72);
		HFONT hOldFont;

		GetVersionEx(&osv);
                
    		MultiByteToWideChar( CP_ACP, 0, szFont,strlen(szFont)+1, fontName, 1024 );
		hFont = CreateFontW(ITEM_POINT_SIZE, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
                   CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, fontName);

                hOldFont = (HFONT)SelectObject(hdc, hFont);
		AdjustDialogSize(hdc,hWnd,hLink,wCmdLine);

 
		ReleaseDC(hLink, hdc);

		SendMessage(hLink,WM_SETFONT,(WPARAM)hFont,TRUE);
    }


   /*
    * position result dialog in middle of the screen
    */
   {
   RECT SystemRect;
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

     char MajorVer[4]={0};
     char MinorVer[4]={0};

   if( Version_1 != 0 ){
     itoa(Version_1 >> 8 ,MajorVer,10);
     itoa(Version_1 & 0x000000FF ,MinorVer,10);

     sprintf((char*)HinstallTitle,NT_PROGRAM_TITLE,MajorVer,MinorVer);
     SetWindowText(hWnd,(char*)HinstallTitle);
     
    }

    hwndOwner = GetDesktopWindow(); 
  
    GetWindowRect(hwndOwner, &rcOwner); 

    GetWindowRect(hWnd, &rcDlg); 
    CopyRect(&rc, &rcOwner); 

    // Offset the owner and dialog box rectangles so that right and bottom 
    // values represent the width and height, and then offset the owner again 
    // to discard space taken up by the dialog box. 

    OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
    OffsetRect(&rc, -rc.left, -rc.top); 
    OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 

    // The new position is the sum of half the remaining space and the owner's 
    // original position. 

    SetWindowPos(hWnd, 
                 HWND_TOP, 
                 rcOwner.left + (rc.right / 2), 
                 rcOwner.top + (rc.bottom / 2), 
                 0, 0,          // Ignores size arguments. 
                 SWP_NOSIZE); 

   }





    return TRUE;
}

LRESULT OnSysLinkNotify(HWND hWnd, int idCtrl, LPNMHDR pNMHdr)
{
    if (idCtrl != IDC_SYSLINK)	// Make sure that it is the SysLink control
        return 0;

    // The notifications associated with SysLink controls are NM_CLICK 
    // (syslink) and (for links that can be activated by the Enter key) 
    // NM_RETURN.
    switch (pNMHdr->code)
    {
       case NM_CLICK:
       case NM_RETURN:
       {
            PNMLINK pNMLink = (PNMLINK)pNMHdr;
            LITEM item = pNMLink->item;

	    // Judging by the index of the link
            ShellExecuteW(NULL, L"open", item.szUrl, NULL, NULL, SW_SHOW);
            break;
        }
    }
    return 0;
}

LRESULT OnSysLinkCommand(HWND hWnd, WPARAM wParam, HWND lParam,int bls)
{
  switch(LOWORD(wParam)) // ID kontrolki
  {
    case IDC_OK: 
	{
	  EndDialog(hWnd, 0);
	  break;
	}
    default : 
        return 0;
  }
  return 0;
}

void OnClose(HWND hWnd)
{
    if( NULL != hFont) DeleteObject(hFont);
    EndDialog(hWnd, 0);
}


void AdjustDialogSize(HDC hdc,HWND hWnd, HWND hLink,WCHAR* String)
{

		//
		// get text height
		//
		SIZE size ={0};
		int height = 0;
		WCHAR* first=NULL,*next=NULL;
		WCHAR* start= String;
		int width = 0;
		int number = 1;
		WCHAR* href_start=NULL;
		int href_len = 0;
		RECT topRect;
		RECT topClientRect;
		RECT syslinkrect;
		POINT p;
		HWND ok;
		RECT myRect;
		int dist_syslink = 20;
		int dist_syslink_ok = 20;
		int wx;
		int wy;

		int newwidth;
		int newheight;

		GetTextExtentPointW(hdc,String,wcslen(String),&size);
		
		height = size.cy;

		// get max width
		
		first = wcschr(	String,'\n');
		if( first == NULL ){
			/*
			 * we do not have \n in the string
			 */ 
            		href_start = wcsstr(start,L"HREF=");
			width = wcslen(start);
			/*
    		 	 * HREF= should not count to string size
			 */
			if( href_start != 0 ){
				href_len = wcschr(start,'>') - href_start;
			}
			width -= href_len;
            		/*
			 * check max line size
			 */
			if( width > MAX_WIDTH ){
				number += width/MAX_WIDTH + 1;
				width = MAX_WIDTH;
			}
		}
		else{
			/*
			 * string contains \n to be displayed on multiple lines.
			 */
			first = String;
			for(;;){
				next = wcschr(first,'\n');
                href_start = wcsstr(first,L"HREF=");
				href_len = 0;

				if( next != NULL ){
					/*
					 * we detected more than \n
					 */
					number++;
					/*
					 * HREF= should not count to string size
					 */
					if( href_start != 0 && href_start < next ){
						href_len = wcschr(first,'>') - href_start;
					}

					if( width < (next-first)) {
						width = next - first;
						start = first;
					}
					first = next+1;
					if( href_start < next )
						width -= href_len;
                                /*
		         	 * check max line size
			         */
					if( width > MAX_WIDTH ){
						number += width/MAX_WIDTH + 1;
						width = MAX_WIDTH;
					}
				}
				else{
					/*
					 * last line in the string 
					 */

					/*
					 * HREF= should not count to string size
					 */
					if( href_start != 0 && href_start > first ){
						href_len = wcschr(first,'>') - href_start;
					}
					else
						href_len = 0;

					if( width < wcslen(first) - href_len){
						width = wcslen(first) - href_len;
						start = first;
					}
					
                    /*
		         	 * check max line size
			         */
					if( width > MAX_WIDTH ){
						number += width/MAX_WIDTH + 1;
						width = MAX_WIDTH;
					}
					break;
				}
			}
		}

		GetTextExtentPointW(hdc,start,width,&size);

		width = size.cx;

		//
		// resize windows
		//

		GetWindowRect(hWnd,&topRect);
		ok = GetDlgItem(hWnd,IDC_OK);

		GetWindowRect(hLink,&syslinkrect);
		GetWindowRect(ok,&myRect);

		p.x = width;
		p.y = (height /*+3*/ ) * (number+1);

		LPtoDP(hdc,&p,1);

		wx = p.x-(syslinkrect.right-syslinkrect.left);
		wy = p.y - (syslinkrect.bottom-syslinkrect.top);

		newwidth = p.x + 2*dist_syslink;
		newheight = topRect.bottom - topRect.top + wy; /*+ 40 + (myRect.bottom-myRect.top) + 20;*/

		MoveWindow(hWnd,topRect.left-wx/2,topRect.top - wy/2,newwidth,newheight,TRUE);
		
		MoveWindow(hLink,dist_syslink,dist_syslink,p.x /*topRect.right-topRect.left-40*/,p.y /*topRect.bottom-topRect.top -(myRect.bottom-myRect.top)*/,TRUE);

		MoveWindow(ok,newwidth/2 - (myRect.right-myRect.left)/2,dist_syslink + p.y + dist_syslink_ok,myRect.right-myRect.left,myRect.bottom-myRect.top,TRUE);



}

